from __future__ import division
import simuPOP
import simuPOP.utils

import collections
import os
import os.path

from generate_genome import *
from simuPOP_functions import *
from plink_functions import *
from Sim_Results_class import Sim_Results

BASE_DIR = 'C:\\Users\\waplesry\\Dropbox\\Projects\\simulate_Ne\\real_data'
#PLINK_DIR = os.path.join(BASE_DIR, 'plink')
PLINK_DIR = BASE_DIR
PLINK_MAF = .05
PLINK_EXE = "C:\\plink\plink-1.07-dos\\plink.exe"

SPID = "C:\\Users\\waplesry\\Dropbox\\Programs\\PGDSpider_2.0.4.0\\genepop_to_map_ped.spid"
PDG_SPIDER_EXE = "C:\\Users\\waplesry\\Dropbox\\Programs\\PGDSpider_2.0.4.0\\PGDSpider2-cli.exe"

class External_Data():
	def __init__(self, file, format = 'genepop', adjust = 0):
		"""
		Loads a genepop file as a simuPOP population, 
		set to self.population
		"""
		external_pop = simuPOP.utils.importPopulation(filename = file, format = format, adjust = adjust)
		self.path  = os.path.split(file)[0]
		self.filename = os.path.split(file)[1]
		self.basename = os.path.splitext(self.filename)[0]
		self.population = external_pop
		
	def filter_MAF(self, maf = 0.05):
		simuPOP.stat(self.population, alleleFreq = simuPOP.ALL_AVAIL)
		loci_to_keep = []
		loci_to_remove = []
		for locus in self.population.vars()['alleleNum'].keys():
			allele_counts = []
			for allele in self.population.vars()['alleleNum'][locus]:
				if allele != 0:
					allele_counts.append(self.population.vars()['alleleNum'][locus][allele])
			if len(allele_counts) > 2:
				raise StandardError("Should not be more than two alleles at locus {}".format(locus))
			allele_ratio = allele_counts[0] / sum(allele_counts) 
			if (allele_ratio > 1-maf) or (allele_ratio < maf):
				loci_to_remove.append(locus)
			else:
				loci_to_keep.append(locus)
		print ("Removing {} loci for MAF < {}".format(len(loci_to_remove), maf))
		print(loci_to_remove)
		self.population.removeLoci(loci = loci_to_remove)			
		
	def filter_missing(self, missing_allele = 0, threshold = 0.1):
		simuPOP.stat(self.population, alleleFreq = simuPOP.ALL_AVAIL)
		loci_to_keep = []
		loci_to_remove = []
		for locus in self.population.vars()['alleleFreq'].keys():
			if self.population.vars()['alleleFreq'][locus][missing_allele] > threshold:
				loci_to_remove.append(locus)
			else:
				loci_to_keep.append(locus)
		print ("Removing {} loci for missingness > {}".format(len(loci_to_remove), threshold))
		print(loci_to_remove)
		self.population.removeLoci(loci = loci_to_remove)
		
	def sample_size_matrix(self):
		"""Generates a numpy.array of the """
		# # assumes that ordering of individuals is maintained
		num_inds = self.population.popSize()
		num_loci = self.population.totNumLoci()
		gg = numpy.array(self.population.genotype())
		hh = numpy.reshape(gg, (num_inds, 2, num_loci)) # inds ploidy, loci
		results_mat = numpy.zeros(shape = (num_loci, num_loci), dtype=numpy.int)
		for ii in range(self.population.totNumLoci()):
			for jj in range(self.population.totNumLoci()):
				#non zero product mean both alleles are present
				results_mat[ii, jj] = numpy.count_nonzero(hh[:,0,ii] * hh[:,0,jj])
		self.S_mat = results_mat			
		
	def export_map_ped(self, adjust = 0):
		simuPOP.utils.export(self.population, output = os.path.join(self.path, self.basename + '.map'), format = 'MAP')
		simuPOP.utils.export(self.population, output = os.path.join(self.path, self.basename + '.ped'), format = 'PED', adjust = adjust)
		
	def export_genepop(self, adjust = 0):
		simuPOP.utils.export(self.population, output = os.path.join(self.path, self.basename + '.gen'), format = 'genepop', adjust = adjust)

	def load_map(self, filename):
		"""
		Load a linkage map file.
		self.map = dict:
			map[locus] = tuple(lg, cM)
		self.lgs = defaultdict:
			lgs[lg] = list of tuples(locus, cM)
		locus = string
		lg = int
		cM = float
		"""
		self.map = dict()
		self.lgs = collections.defaultdict(list)
		with open(filename, 'r') as INFILE:
			first_line = next(INFILE).strip()
			if first_line != 'tag\tlg\tcM':
				raise StandardError("First line of map file should be 'tag[tab]lg[tab]cM'")
			for line in INFILE:
				str_locus, str_lg, str_cM = line.strip().split()
				if str_locus in self.population.lociNames():
					int_lg = int(str_lg)
					float_cM = float(str_cM)
					self.map[str_locus] = int_lg, float_cM
					self.lgs[int_lg].append((str_locus, float_cM))
				
	def map_to_c_matrix(self, filename = None):
		cM_genome = []
		total_loci = len(self.map)
		for lg in sorted(self.lgs.keys()):
			cM_chromosome = sorted([x[1] for x in self.lgs[lg]])
			cM_genome += [cM_chromosome]
		# start with c = 0.5
		initial_matrix = numpy.array([0.5 for xx in range(total_loci**2)])
		initial_matrix = numpy.reshape(a = initial_matrix, newshape = (total_loci, total_loci))
		start_pos = 0 
		last_pos = 0
		# Fill in values within chromosomes
		for each_chr in range(len(cM_genome)):
			partial_c_matrix = chromosome_c_matrix(cM_genome[each_chr], unit = 1)
			last_pos = start_pos + len(cM_genome[each_chr])
			initial_matrix[start_pos:last_pos,start_pos:last_pos] = partial_c_matrix
			start_pos = last_pos
		self.c_matrix = initial_matrix
		if filename is not None:
			#numpy.savetxt(fname = filename, X = self.c_matrix, delimiter = "\t", fmt = '%1.6f')
			numpy.save(file = filename, arr = self.c_matrix)
	#  S should actually be a function of the number of genotypes shared by both loci.
	
	def expected_r2_WH(self, Ne, S):
		#S = self.population.subPopSize()
		self.r2_WH = expected_r2_WH(self.c_matrix, Ne, S)
		
	def expected_r2_SF(self, Ne, S):
		#S = self.population.subPopSize()
		self.r2_SF = expected_r2_WH(self.c_matrix, Ne, S)
	
	def expected_r2_lower(self):
		lower_matrix = numpy.empty(self.r2_WH.shape)
		lower_matrix[self.r2_SF < self.r2_WH] = self.r2_SF[self.r2_SF < self.r2_WH]
		lower_matrix[self.r2_WH <= self.r2_SF] = self.r2_WH[self.r2_WH <= self.r2_SF]
		self.r2_lower = lower_matrix

	# Get Folded Allele Frequencies
	def set_folded(self):
		simuPOP.stat(self.population, alleleFreq = simuPOP.ALL_AVAIL)
		self.folded = get_folded_freqs(self.population)
		filename = os.path.join(self.path, self.basename + ".fold")
		with open(filename, 'w') as OUTFILE:
				OUTFILE.write("\n".join(str(xx) for xx in self.folded))
