from __future__ import division
import numpy
import scipy.spatial.distance
import simuPOP 
import simuPOP.utils
import os.path
import matplotlib
import collections

from simuPOP_functions import read_R2_matrix
from generate_genome import expected_r2_WH, expected_r2_SF #, expected_r2_Park, est_c_numerator
from PARAMS import DEBUG

import logging
import log_config
Results_slass_log = logging.getLogger('Results_class')


# TODO 3/2/14
# Include some 'correction' regimes in the reprted results.

# Methods that filter based on r^2  
 # 1 - cutoff r^2, have a blank r^2 cuttof, say 0.5.
 # 2 - flat r^2 fraction, remove the hightest X% of the comparisons, say 1%
 # 3 - adaptive r^2 fraction, remove the highet X%of comparisons, as a function of # of chromosomes
        # try a function of    1 / X )

#Methods that work on the distribution as a whole. 
 # 4 - leverage the chi^2 distribution to adjust the mean r^2. 
 # Rather than accepting or rejecting each value, build an 'expected' chi^2 distribution to compare to.
 # Use the mean r^2 initially.
    # generate an expected chi^2
    # compare this distribution to the observed data.
    # in each bin, adjeust r2 values based on the ratio of expected to observed data points.
    # so if you see 20x more than expected, adjust each value by 1/20
    # slightly depend on bin size. 
  # match the 'slope' of the observed distribution to a corresponding neutral chi2 dist., to try to guess it.

# Meta-analysis - compare across GENOME replicates to be very conservative about what r^2 would look like across related populations, 
    # that share a GENOME but without a shared genetic history.




class Results():
	def set_genetics(self, directory, name, ext = '.gen', format = 'genepop', adjust = 0):
       	    """
       	    set self.genetics to a simuPOP population from the given genepop file.
       	    only GENEPOP files are supported
       	    """
       	    filename = os.path.join(directory, name + ext)
       	    self.genetics = simuPOP.utils.importPopulation(filename = filename, format = format, adjust = adjust)
	
	def set_r2_plink(self, directory, name, ext = '.plinkR2.gz'):
		self.r2_plink = read_R2_matrix(os.path.join(directory,name + ext))
		return(True)
	
	# Removed 1/8/14
	# replaced by set_genome / set_retained_genome	
	#def set_c_mat(self, directory, name, ext = '.c_mat.gz'):
	#	self.c_mat = read_R2_matrix(os.path.join(directory, name + ext))
	#	return(True)
		
	def set_genome(self, directory, genome_name, ext = '.genome.tsv.gz'):
	    self.whole_genome = scipy.spatial.distance.squareform(read_R2_matrix(os.path.join(directory, genome_name + ext)))
	    return(True)
	    
	def subset_genome(self, name, directory, ext = '.include'):
	   with open(os.path.join(directory, name + ext)) as INFILE:
	       included_loci = [int(line.strip()) for line in INFILE]
	       #select by the rows and columns in the file.
	       self.retained_genome = self.whole_genome[:,included_loci][included_loci,:]
	
	def get_genome_from_name(self, name):
	    name_split = name.split("+")
	    for sec in name_split:
	        if "genome_" in sec:
	            genome_string = sec.split("_")[1]
	    return(genome_string)
	
	def set_retained_genome(self, name, base_directory):
	    genome_name = self.get_genome_from_name(name = name)
	    self.set_genome(genome_name = genome_name, directory = os.path.join(base_directory, 'genomes'))
	    self.subset_genome(name = name, directory = os.path.join(base_directory, 'samples', 'loci'))	    
	    
	                 	
	def set_expected_r2(self, name, base_directory, sample_size, popsize, write_WH_SF_exp_r2 = False):
		ssc = 1/float(sample_size)
		
		# Weir and Hill
		r2_WH = expected_r2_WH(c = self.retained_genome, Ne = popsize) + ssc
		Results_slass_log.debug("Created WH exp r2 matrix")
		# Sved and Fledman Method
		r2_SF = expected_r2_SF(c = self.retained_genome, Ne = popsize) + ssc
		Results_slass_log.debug("Created SF exp r2 matrix")
			
		# Review the target directory
		if write_WH_SF_exp_r2 is True:
		  #numpy.savetxt(X = r2_WH, fname = os.path.join(base_directory, 'samples', 'expected_r2', name + '.WH_r2_mat.gz'), delimiter = "\t", fmt = '%1.6f')
		  #numpy.savetxt(X = r2_SF, fname = os.path.join(base_directory, 'samples', 'expected_r2', name + '.SF_r2_mat.gz'), delimiter = "\t", fmt = '%1.6f')
		  numpy.save(arr = r2_WH, file = os.path.join(base_directory, 'samples', 'expected_r2', name + '.WH_r2_mat.gz'))
		  numpy.save(arr = r2_SF, file = os.path.join(base_directory, 'samples', 'expected_r2', name + '.SF_r2_mat.gz'))			
			
		  Results_slass_log.debug("WH, SF exp r2 files written to path: {}".format((base_directory, 'samples', 'expected_r2', name + "*_r2_mat.gz")))
		
		# Calculate the lower of the two estimates
		lower_matrix = numpy.empty(r2_WH.shape)
		lower_matrix[r2_SF < r2_WH] = r2_SF[r2_SF < r2_WH]
		lower_matrix[r2_WH <= r2_SF] = r2_WH[r2_WH <= r2_SF]
		#numpy.savetxt(X = lower_matrix, fname = os.path.join(base_directory, 'samples', 'expected_r2', '{}.lower_r2_mat.gz'.format(name)), delimiter = "\t", fmt = '%1.6f')		numpy.savetxt(X = lower_matrix, fname = os.path.join(base_directory, 'samples', 'expected_r2', '{}.lower_r2_mat.gz'.format(name)), delimiter = "\t", fmt = '%1.6f')
		numpy.save(arr = lower_matrix, file = os.path.join(base_directory, 'samples', 'expected_r2', '{}.lower_r2_mat.gz'.format(name)))
		Results_slass_log.debug("exp r2 matrix written to file: {}".format(os.path.join(base_directory, 'samples', 'expected_r2', '{}.lower_r2_mat.gz'.format(name))))
		self.expected_r2 = lower_matrix
		return(True)
			
	def set_expected_r2_unlinked(self, popsize, sample_size):
		c = .5
		Ne = popsize
		exp_SF = expected_r2_SF(c = c, Ne = Ne, bound = True)
		exp_WH = expected_r2_WH(c = c, Ne = Ne, bound = True)
		expected_r2_unlinked = min(exp_SF, exp_WH)
		expected_r2_unlinked = expected_r2_unlinked + 1.0/sample_size
		self.expected_r2_unlinked = expected_r2_unlinked
		return(True)

	def set_AFS(self, directory, name, ext = '.fold'):
	   """set self.AFS to be a numpy array of MAFs"""
	   AFS = numpy.loadtxt(fname = os.path.join(directory, name + ext))
	   self.AFS = AFS
	
	def set_AFS_frac_lowest_bin(self):
	   self.AFS_frac_lowest_bin = sum(self.AFS < .1) / float(sum(self.AFS > -1))
		
	def set_fraction_c_point5(self):
	   working = scipy.spatial.distance.squareform(self.retained_genome)
	   self.fraction_c_point5 = sum(working == .5) / float(sum(working < 2))

	def set_Ne_estimates(self):
		#self.median_r2_of_subset = dict()
		self.r2_of_subset = dict()
		self.Ne_of_subset = dict()
		self.pairs_of_subset = dict()	
		# Weir's sampling bias correction
		S = self.sim_params['sample_size']
		Weir_sbc = (float(S)/(S-1))**2
		Results_slass_log.debug("Weir's sampling bias correction: {}".format(Weir_sbc))
	       
		for subset_key in self.subset_loci.keys():
		  subset = self.subset_loci[subset_key]
		  Results_slass_log.debug("Subset_shape: {}, r2 shape: {}".format(subset.shape, self.r2_plink.shape))
		  mean_r2 = numpy.mean(self.r2_plink[self.subset_loci[subset_key]]) * Weir_sbc
		  Ne = self.estimate_Ne_LDNe(mean_r2, S)
		  self.r2_of_subset[subset_key] = mean_r2
		  self.Ne_of_subset[subset_key] = Ne
		  self.pairs_of_subset[subset_key] = int(self.r2_plink[self.subset_loci[subset_key]].shape[0])
		
	# Subsetting by c
	def subset_by_c(self):
		self.subset_loci = dict()
		self.set_index_array_c('all', 0, 0.50001)
		self.set_index_array_c('c==0.5', 0.49999, 0.50001)
		#self.set_index_array_c('c>=0.1', 0.1, 0.50001)
		#self.set_index_array_c('c<=0.001', 0, 0.0011)
	
	def set_index_array_c(self, key, c_min, c_max):
	   working = scipy.spatial.distance.squareform(self.retained_genome)
	   indexing_array = numpy.logical_and(working >= c_min, working <= c_max)
	   self.subset_loci[key] = indexing_array

	# Analysis
	def estimate_Ne_LDNe(self, mean_r2, S):
		r2_sample = (1.0/S) + 3.19/(S*S)
		r2_prime = mean_r2 - r2_sample
		if 1/9.0 - 2.76*r2_prime > 0:
			numerator = 1/3.0 + numpy.sqrt(1/9.0 - 2.76*r2_prime)
			denominator = 2*r2_prime
			Ne = (numerator/denominator)
		else:
			Ne = (1/3.0) * (2* r2_prime)
		return(Ne)
	
	def set_expected_r2_distribution(self, subset_key = None):
		c_matrix = self.retained_genome
		if subset_key is None:
			pass
		else:
			c_matrix = c_matrix[self.subset_loci[subset_key]]
	
	def r2_difference(self, obs_r2, exp_r2, subset_key = None):
		if subset_key is None:
			return(obs_r2 - exp_r2)
		else:
			return(obs_r2[self.subset_loci[subset_key]] - exp_r2[self.subset_loci[subset_key]])
			
	def plot_r2_difference(self, subset_key = None):			
		obs_r2 = self.r2_plink
		exp_r2 = self.expected_r2
		r2_diff = self.r2_difference(obs_r2 = obs_r2, exp_r2 = exp_r2, subset_key = subset_key)
		matplotlib.pyplot.hist(r2_diff, 
			log = True, 
			bins = 100, 
			range = (-1,1)
			)
		matplotlib.pylab.title("Mean Difference: {}\n{} Subset = {}".format(numpy.mean(r2_diff), self.name, subset_key))
		matplotlib.pylab.xlabel('R2 diffence')
		matplotlib.pylab.ylabel('Count')
	
	def plot_r2_distribution(self, subset_key = None):
		obs_r2 = self.r2_plink
		exp_r2 = self.expected_r2
		if subset_key is None:
			pass
		else:
			obs_r2 = obs_r2[self.subset_loci[subset_key]]
			exp_r2 = exp_r2[self.subset_loci[subset_key]]
		matplotlib.pyplot.hist(obs_r2, log = True, bins = 50, range = (0,1), alpha = .5, color = "Blue")
		matplotlib.pyplot.hist(exp_r2, log = True, bins = 50, range = (0,1), alpha =.5, color = "Green")
		matplotlib.pylab.xlim(0, 1)
		matplotlib.pylab.ylim(ymin = 1)
		matplotlib.pylab.title("Blue = Observed / Green = Expected\n{}, Subset = {}".format(self.name, subset_key))
		matplotlib.pylab.ylabel("Count")
		matplotlib.pylab.xlabel("R2")
		
	# Observed vs Expected R2
	def plot_obs_exp_r2(self, subset_key = None):
		obs_r2 = self.r2_plink
		exp_r2 = self.expected_r2
		if subset_key is None:
			pass
		else:
			obs_r2 = obs_r2[self.subset_loci[subset_key]]
			exp_r2 = exp_r2[self.subset_loci[subset_key]]
			
		matplotlib.pyplot.hexbin(exp_r2, obs_r2,
			mincnt = 1, bins = 'log', gridsize = 30)
		matplotlib.pylab.ylim([-0.04, 1.04])
		matplotlib.pylab.xlim([-0.04, 1.04])
		matplotlib.pylab.title("exp R2 vs OBS R2\n{}, Subset = {}".format(self.name, subset_key))
		matplotlib.pylab.xlabel('Lower of [SF, WH] expected R2 ')
		matplotlib.pylab.ylabel('Observed R2')
		cb = matplotlib.pyplot.colorbar()
		cb.set_label('log(count)')
		matplotlib.pyplot.show()
	# Observed R2 vs Recombination Fraction
	def plot_c_obs_r2(self, subset_key = None, xscale = 'linear'):
		c_mat = self.retained_genome
		obs_r2 = self.r2_plink
		if subset_key is None:
			pass
		else:
			c_mat = c_mat[self.subset_loci[subset_key]]
			obs_r2 = obs_r2[self.subset_loci[subset_key]]
		matplotlib.pyplot.hexbin(c_mat, obs_r2,
			mincnt = 1, bins = 'log', gridsize = 60,
				xscale = xscale
			 )
		matplotlib.pylab.ylim([-0.04, 1.04])
		matplotlib.pylab.xlim([-0.02, 0.52])
		matplotlib.pylab.title("c vs Observed R2\n{}, Subset = {}".format(self.name, subset_key))
		matplotlib.pylab.xlabel('Recombination Fraction (c)')
		matplotlib.pylab.ylabel('Observed R2')
		cb = matplotlib.pyplot.colorbar()
		cb.set_label('log(count)')
		matplotlib.pyplot.show()
		
	def plot_r2_hist(self, subset_key = None):
		obs_r2 = self.r2_plink
		if subset_key is None:
			pass
		else:
			obs_r2 = obs_r2[self.subset_loci[subset_key]]
		matplotlib.pyplot.hist(obs_r2, log = True)
			

class Sim_Results(Results):
	def __init__(self, directory, name, generate_expected_r2 = False):
		self.name = name
		self.directory = directory
		self.set_sim_params()
		self.set_genetics(directory = os.path.join(self.directory, 'samples', 'genepop'), name = self.name)
		self.set_r2_plink(directory = os.path.join(self.directory, 'samples', 'plink', 'output'), name = self.name)
		self.set_retained_genome(base_directory = self.directory, name = self.name)
		if generate_expected_r2:
		  self.set_expected_r2(directory = os.path.join(self.directory, 'samples', 'expected_r2'), name = self.name)
		self.set_expected_r2_unlinked(popsize = self.sim_params['popsize'], sample_size = self.sim_params['sample_size'])
		self.set_AFS(directory = os.path.join(self.directory, 'samples', 'fold'), name = self.name)
		self.subset_by_c()
		self.set_AFS_frac_lowest_bin()
		self.set_fraction_c_point5()
		self.set_Ne_estimates()
		
	def set_sim_params(self):
		"""Gets simulation parameters from the file name"""
		#popsize, sample_size, generation, loci_num, chr_num, chr_cM, scale_recombination, scale_mutation, rep = self.name.split("_")
		first_splits = self.name.split("+")
		if DEBUG: print("Parse simulation parameters; {} ".format(first_splits))
		popsize , loci_num, chr_num, chr_cM, cM_unit, ascertainment_AF, current_genome, sample_size, generation, simuPOP_rep, SIMCOAL_rep, sampling_rep  = ((xx.split("_"))[1] for xx in first_splits)
		self.sim_params = dict()
		self.sim_params['popsize'] = int(popsize)
		self.sim_params['loci_num'] = int(loci_num)
		self.sim_params['chr_num'] = int(chr_num)
		self.sim_params['chr_cM'] = float(chr_cM)
		self.sim_params['cM_unit'] = int(cM_unit)
		self.sim_params['ascertainment_AF'] = float(ascertainment_AF)
		self.sim_params['current_genome'] = "-".join([str(jj) for jj in current_genome])
		self.sim_params['sample_size'] = int(sample_size)
		self.sim_params['generation'] = int(generation)
		self.sim_params['simuPOP_rep'] = int(simuPOP_rep)
		self.sim_params['SIMCOAL_rep'] = int(SIMCOAL_rep)
		self.sim_params['sampling_rep'] = int(sampling_rep)
		return(True)

	def summary(self, *args):
		summary = collections.OrderedDict()
		#sim parameters
		summary['popsize'] =  self.sim_params['popsize']
		summary['starting_loci'] =  self.sim_params['loci_num']
		summary['chr_num'] =  self.sim_params['chr_num']
		summary['chr_cM'] =  self.sim_params['chr_cM']
		summary['cM_unit'] =  self.sim_params['cM_unit']
		summary['ascertainment_AF'] = self.sim_params['ascertainment_AF']
		summary['sample_size'] =  self.sim_params['sample_size']
		summary['generation'] =  self.sim_params['generation']
		summary['current_genome'] =  self.sim_params['current_genome']
		summary['SIMCOAL_rep'] =  self.sim_params['SIMCOAL_rep']
		summary['simuPOP_rep'] =  self.sim_params['simuPOP_rep']
		summary['sampling_rep'] =  self.sim_params['sampling_rep']
		
		#Final Loci
		summary['final_loci '] =  len(self.AFS)
		
		#AFS
		summary['AFS_lowest_bin'] =  self.AFS_frac_lowest_bin
		
		# Recombination
		summary['fraction_c_point5'] =  self.fraction_c_point5
		summary['min_c'] =  numpy.min(scipy.spatial.distance.squareform(self.retained_genome))
		
		# mean r2
		summary['mean_r2_all']   = self.r2_of_subset['all']
		#summary['mean_r2_almost_unlinked']  = self.r2_of_subset['c>=0.1']
		summary['mean_r2_unlinked']  = self.r2_of_subset['c==0.5']
		#summary['mean_r2_closely_linked']  = self.r2_of_subset['c<=0.001']
		# median r2
		# removed 12/9/13
		#summary['median_r2_all']   = self.median_r2_of_subset['all']
		#summary['median_r2_almost_unlinked']  = self.median_r2_of_subset['c>=0.1']
		#summary['median_r2_unlinked']  = self.median_r2_of_subset['c==0.5']
		#summary['median_r2_closely_linked']  = self.median_r2_of_subset['c<=0.001']
		
		# Ne 
		summary['Ne_all']   = self.Ne_of_subset['all']
		#summary['Ne_almost_unlinked']  = self.Ne_of_subset['c>=0.1']
		summary['Ne_unlinked']  = self.Ne_of_subset['c==0.5']
		#summary['Ne_closely_linked']  = self.Ne_of_subset['c<=0.001']
		
		# pairs of subset
		summary['pairs_all']  = self.pairs_of_subset['all']
		#summary['pairs_almost_unlinked']  = self.pairs_of_subset['c>=0.1']
		summary['pairs_unlinked']  = self.pairs_of_subset['c==0.5']
		#summary['pairs_closely_linked']  = self.pairs_of_subset['c<=0.001']
		return(summary)
				
	
class Real_Data_Results(Results):
	def __init__():
	    #not yet implemented
	    pass

	


	

	




