#chi2 correction
import numpy
from scipy.stats import chi2

def generate_bins(interval = 0.01, xmax = 10):
    left_intervals = numpy.arange(0, xmax, interval)
    right_intervals = numpy.arange(interval, xmax + interval, interval)	
    at_or_above_one = right_intervals >= 1.0
    at_one = right_intervals == 1.0
    # pack bins
    bins = left_intervals, right_intervals, at_or_above_one, at_one
    return(bins)


def find_chi2_of_r2(r2, num_obs, bins):
    """
    Returns the chi2 distribution that describes the expected counts of
    each r2 bin given an r2 value and number of observations.
    As if the r2 with have a mean = r2
    returns: tuple(intervals, counts)
    """
    # unpack bins
    left_intervals, right_intervals, at_or_above_one, at_one = bins
    #midpoint of each bin
    intervals = (right_intervals + left_intervals)/2. 
    
    # Generate Probability distribution
    scaled_chi2_dist_frozen = chi2(1, scale = r2)
    scaled_left_cdf = scaled_chi2_dist_frozen.cdf(left_intervals)
    scaled_right_cdf = scaled_chi2_dist_frozen.cdf(right_intervals)
    scaled_interval_density = scaled_right_cdf - scaled_left_cdf
    
    # Scale prob distribution
    r2_cumulative_density = scaled_interval_density * num_obs
    
    # Adjust for boundary at one
    density_above_one = numpy.sum(r2_cumulative_density[at_or_above_one])
    r2_cumulative_density[at_or_above_one] = 0
    r2_cumulative_density[at_one] = density_above_one
    
    return(intervals, r2_cumulative_density)

