
# Goal
# We need a way to generate; 
	# clumped 
	# random 
# distributions of loci across a genome.

import scipy.stats
import random

# Example of a highly clumped spacing
#negative binomial (n = number trials, p = prob)
# mean = n(1 - p)/p
# variance = n(1-p)/ (p^2) = mean + 1/n*(mean^2)
nb = scipy.stats.nbinom(1, .002)

def generate_cM_chromosome(chr_cM, chr_loci, cM_unit):
	"""
	Returns a numpy.array of *num_loci* loci onto a chromosome *num_cM* units long.
	"""
	if chr_cM <= 0:
		raise Exception("Please select a positive cM as the chromosome length")
	else:
		positions = sorted(random.sample(range(int(chr_cM * cM_unit)), chr_loci))
		cM_chromosome = numpy.array([xx/float(cM_unit) for xx in positions])
		return (cM_chromosome)	

rand_pos = generate_cM_chromosome(100, 200, 1000)
figure(3)
vlines(rand_pos, [-.1]*200,[.1]*200, linewidths  = 2, alpha = .5)
ylim(-1, 1)
xlim(-5, numpy.max(rand_pos)+5)
xlabel("cM")
figure(4)
hist(numpy.diff(rand_pos), bins = 100)

bn = scipy.stats.binom(100, .99)


#draws = nb.rvs(size = 199)
#hist(draws)#, bins = range(0,50))
#title(nb.stats(moments='mv'))
#xlabel("Total cM: {}".format(numpy.sum(draws)/1000.))


def plot_chr(dist, num_loci):
    draws = dist.rvs(size = num_loci)
    figure(1)
    hist(draws/1000., bins = 1000)
    title(dist.stats(moments='mv'))
    xlabel("Total cM: {}".format(numpy.sum(draws)/1000.))
    figure(2)
    placements = numpy.cumsum(draws)/1000.
    vlines(placements, [-.1]*num_loci,[.1]*num_loci, linewidths  = 2, alpha = .5)
    ylim(-1, 1)
    xlim(-5, numpy.max(placements)+5)
    xlabel("cM")
    

plot_chr(nb, 199)

# for each of these spots assign a recombination rate.
# option one 
    # number of inter-locus spacings
    # a number of hot_cold_spots (each)
    # an expansion / contraction ratio of recombination rate.
    
def hot_cold(n, num_spots, ratio = 50, width = 50):
    """
    n = total number of spaces between possible locus slots
    hot_cold = number of hot and cold spots (each)
    ratio = ratio of rates to background rate
    background = 1
    cold = 1/ratio
    hot = ratio
    for each in n, assign a recom rate. """
    spots = numpy.arange(0, n/width, 1)
    rates = numpy.array([1.]*n)
    target_spots = numpy.random.choice(spots, size = num_spots*2, replace = False)
    cold_spots = target_spots[0:num_spots]
    hot_spots = target_spots[num_spots:num_spots*2]
    for xx in cold_spots:
        start_spot = xx * width
        for jj in range(0, 50):
            rates[start_spot+jj] = 1./ratio
    for xx in hot_spots:
        start_spot = xx * width
        for jj in range(0, 50):
            rates[start_spot+jj] = 1.*ratio
    rates = rates * (n/numpy.sum(rates)) # re-scale
    return(rates)
    
n = 99999
ini = hot_cold(n, num_spots = 5, ratio = 100, width = 10)
#plot(numpy.arange(0, n), numpy.cumsum(ini) )
#xlabel("Physical")
#ylabel("Recombination")

cM_spots = numpy.insert(arr = numpy.cumsum(ini)/1000, obj = 0, values = 0)
positions = sorted(random.sample(range(int(n+1)), 200))
# actual cM positions of the loci.
spots = cM_spots[positions]
figure(1)
plot(numpy.arange(0, 200), spots, color = 'blue', lw = 3, alpha = .7, label = "Hotspots")
# basic random spacing
ini2 = generate_cM_chromosome(100, 200, 1000)
plot(numpy.arange(0, 200), ini2,  color = 'green', lw = 3, alpha = .7, label = "Random")
plot(numpy.arange(0, 200), numpy.arange(0, 100, .5),  color = 'black', alpha = .5)
xlabel("Locus number")
ylabel("cM position")
legend(loc = "upper left")

figure(2)
hist(numpy.diff(spots),color = 'blue', alpha = .4, bins = numpy.arange(0, 4, .05))
hist(numpy.diff(ini2),color = 'green', alpha = .4, bins = numpy.arange(0, 4, .05))


def cM_to_c(cM, map_function):
	"""Takes as input a cM distance and a map function ["kosambi", "haldane", "etc.."] 
	and returns the recombination fraction (c) as calculated by the map_function.
	Currently only map_function = "kosambi" is supported.
	Kosambi Citation:
		Genetics. 1996 Apr;142(4):1369-77.
		On genetic map functions.
		Zhao H, Speed TP."""
	if map_function == "kosambi":
		r_frac = (.5) * numpy.tanh(2*cM/100.0)
		return (r_frac)
	else:
		raise Exception("Only kosambi is supported")

# distribution changes a bit within a chromosome with the addition of hot /cold spots
figure(3)
cM_chrom = numpy.vstack(tuple(spots))
square_cM_matrix = scipy.spatial.distance.cdist(cM_chrom, cM_chrom)
hot_square_c_matrix = cM_to_c(square_cM_matrix, map_function = 'kosambi')
hist(square_c_matrix.flatten(), alpha = .5, color = 'blue')
cM_chrom = numpy.vstack(tuple(ini2))
square_cM_matrix = scipy.spatial.distance.cdist(cM_chrom, cM_chrom)
rand_square_c_matrix = cM_to_c(square_cM_matrix, map_function = 'kosambi')
hist(square_c_matrix.flatten(), alpha = .5,  color = 'green')


reps = 100
lines_1_y = 0
lines_2_y = 0
for xx in range(reps):
    #hotspot
    ini = hot_cold(n, 500, ratio = 5000)
    cM_spots = numpy.insert(arr = numpy.cumsum(ini)/1000, obj = 0, values = 0)
    positions = sorted(random.sample(range(int(n+1)), 200))
    spots = cM_spots[positions]
    cM_chrom = numpy.vstack(tuple(spots))
    square_cM_matrix = scipy.spatial.distance.cdist(cM_chrom, cM_chrom)
    hot_square_c_matrix = cM_to_c(square_cM_matrix, map_function = 'kosambi')
    #random
    ini2 = generate_cM_chromosome(100, 200, 1000)
    cM_chrom = numpy.vstack(tuple(ini2))
    square_cM_matrix = scipy.spatial.distance.cdist(cM_chrom, cM_chrom)
    rand_square_c_matrix = cM_to_c(square_cM_matrix, map_function = 'kosambi')
    lines_1_rep = hist(scipy.spatial.distance.squareform(hot_square_c_matrix), log = True, bins = numpy.arange(0, .52, .02), visible = False)
    lines_2_rep = hist(scipy.spatial.distance.squareform(rand_square_c_matrix), log = True, bins = numpy.arange(0, .52, .02), visible = False)
    lines_1_y += lines_1_rep[0]
    lines_2_y += lines_2_rep[0]
lines_1 = hist(scipy.spatial.distance.squareform(hot_square_c_matrix), log = True, bins = numpy.arange(0, .52, .02), visible = False)
lines_2 = hist(scipy.spatial.distance.squareform(rand_square_c_matrix), log = True, bins = numpy.arange(0, .52, .02), visible = False)


figure(4)
plot(lines_1[1][1:], lines_1_y/numpy.float(numpy.sum(lines_1_y)), '-', color = 'blue', lw  = 3, alpha = .4, label = "Hot Spots")
plot(lines_2[1][1:], lines_2_y/numpy.float(numpy.sum(lines_2_y)), '-', color = 'green', lw  = 3, alpha = .4, label = "Random")
ylabel('Fraction of Loci')
xlabel('Recombination Fraction')
legend(loc = "lower left")


len(lines_1_y)

# a bit less clumpy, (but doesn't provide zeros!)()
nb = scipy.stats.nbinom(5, .00992)
draws = nb.rvs(size = 199)
hist(draws)#, bins = range(0,50))
title(nb.stats(moments='mv'))
xlabel(numpy.sum(draws))


# less clumpy
nb = scipy.stats.nbinom(50, .1)
draws = nb.rvs(size = 199)
hist(draws)#, bins = range(0,50))
title(nb.stats(moments='mv'))
xlabel(numpy.sum(draws))


# (too) even spacing
nb = scipy.stats.nbinom(500, .5)
draws = nb.rvs(size = 199)
hist(draws)#, bins = range(0,50))
title(nb.stats(moments='mv'))
xlabel(numpy.sum(draws))


n_vals = numpy.arange(1, 100, 5)
p_vals = numpy.arange(.05, .7, .05)

for xx in p_vals:
    plot_x = n_vals
    plot_y = n_vals * ((1-xx)/xx)
    plot(plot_x, plot_y, label = xx)
    ylabel("mean")
    legend(loc = 'upper left')