#!/usr/bin/env python
import glob
import os
import os.path
import subprocess
import gzip
import numpy

from PARAMS import SCRATCH_DIR, BASE_DIR

import logging
import log_config
PLINK_log = logging.getLogger('PLINK')

def batch_plink_r2(glob_string, plink_exe, maf = .05, zip_output = True):
	ped_files = glob.glob(glob_string)
	PLINK_log.info("Found {} PLINK input files".format(len(ped_files)))
	for count, path in enumerate(ped_files):
            # find those already completed
            completed = glob.glob(os.path.join(BASE_DIR, 'samples', 'plink', 'output', "*.plinkR2.npy"))
            # look for current in completed
            name = os.path.splitext((os.path.basename(path)))[0]
            if os.path.join(BASE_DIR, 'samples', 'plink', 'output', "{}.plinkR2.npy".format(name)) not in completed:          
		plink_r2(path, maf = maf, plink_exe = plink_exe, zip_output = zip_output)
		PLINK_log.info("Completed {}".format(count+1))
	    else:
	        PLINK_log.info("Found {} already completed".format(name))
	
	
def plink_r2(file, plink_exe, maf, zip_output, scratch_dir = SCRATCH_DIR):
	"""
	Executes Plink to create a square, pairwise R2 matrix for the target PED/MAP file pair
	Only includes loci with at least a minor allele frequency of *maf*
	"""
	origWD = os.getcwd() # remember our original working directory
        try:
            os.chdir(SCRATCH_DIR)
            rootname = os.path.splitext(os.path.basename(file))[0]
            root = os.path.join(os.path.dirname(file), rootname)
            to_exec = '"{}" --file "{}" --maf {} --map3 --r2 --matrix'.format(plink_exe, root, maf)
            PLINK_log.debug("PLINK command: {}".format(to_exec))
            subprocess.call(to_exec, shell = True)
            new_name = os.path.join(BASE_DIR, 'samples', 'plink', 'output', "{}.plinkR2".format(rootname))
            os.rename("plink.ld", new_name)
            PLINK_log.debug("PLINK completed, gzip output = {} ".format(zip_output))
            if zip_output is True:
		IN_ARR = numpy.loadtxt(fname = new_name)
	        #with open(new_name) as INFILE:
	        #  with gzip.open(filename = new_name + ".gz", mode = 'wb', compresslevel = 5) as OUTZIP:
		#    OUTZIP.writelines(INFILE)
		#with open(name = new_name + ".npy", mode = 'w') as OUTFILE:
		numpy.save(new_name + ".npy", IN_ARR)
		PLINK_log.debug("gzip file name: {}.npy".format(new_name))
		os.remove(new_name)
        except:
            os.chdir(origWD)
            raise
	
# def rename_MAP_files(glob_string):
	# """Renames files matching the glob string by removing their last four characters,
	# meant to remove the .txt extension from .map.txt files """
	# map_files = glob.glob(glob_string)
	# files_seen = len(map_files)
	# files_renamed = 0
	# for path in map_files:
	# # remove the last four characters, leaving just the '.map' extension.
		# os.rename(path, path[:-4])
		# files_renamed += 1
	# print("Renamed {} of {} files found".format(files_renamed, files_seen))
	# return(None)	