#!/usr/bin/env python
import os.path
import pandas as pd
import numpy as np
import scipy.stats as stats


# TODO:
# clean up extra spaces between column names in summary.tsv - DONE
# remove the median columns - DONE
# remove almost and nearly linkage columns. - DONE
# convert to a 'MultiIndex' formatted index for hierarchy? - SEE BELOW
# review dropped columns


# sumamry file
summary_file = os.path.normpath("C:\Users\IPSEG\Desktop\Waples\Dropbox\Projects\simulate_Ne\sim_data/reports/dec-6b\summary.tsv")

# open summary file
#summary_df = pd.DataFrame.from_csv(path = summary_file, sep = "\t", parse_dates = False)
summary_df = pd.read_csv(summary_file, sep = "\t", parse_dates = False)

# drop some columns
summary_df = summary_df.drop(
    ['acertainment_AF',
    'fraction_c_point5',
    "AFS_lowest_bin ",
    'min_c',                        
    'mean_r2_almost_unlinked',     
    'mean_r2_closely_linked',      
    'median_r2_all',               
    'median_r2_almost_unlinked',    
    'median_r2_unlinked',           
    'median_r2_closely_linked',     
    'Ne_almost_unlinked',          
    'Ne_closely_linked',            
    'pairs_almost_unlinked',       
    'pairs_closely_linked', ], axis = 1)       

# Examine the effects of each genome_rep (collapse simcoal + simuPOP)
to_groupby_genome = ["popsize", 'starting_loci', 'chr_num', "chr_cM", "sample_size", 
    "genome_rep"]
grouped_genome = summary_df.groupby(to_groupby_genome, axis = 0)    
            
#determine best ylim
#global_ymin = 1
#global_ymax = 0
#for name, group in grouped:
#    local_ymin = np.min(group.mean_r2_unlinked)
#    local_ymax = np.max(group.mean_r2_unlinked)
#    if local_ymin < global_ymin: global_ymin = local_ymin
#    if local_ymax > global_ymax: global_ymax = local_ymax  
    
mean_r2_values = list()
for name, group in grouped_genome:
    #figure()
    #scatter(group.SIMCOAL_rep, group.mean_r2_unlinked, alpha = .5)
    #title(zip(to_groupby, name))
    #xlabel("SIMCOAL_rep")
    #ylabel("mean r^2 unlinked")
    #ylim(global_ymin * .98, global_ymax * 1.02)
    mean_r2_values.append(group.mean_r2_unlinked)
figure()
boxplot(x = mean_r2_values, notch = False, sym = "rD" )
xlabel("Genome Replicate")
ylabel("mean r^2")
title(zip(to_groupby_genome, [500, 400, 10, 100, 50, 4 ]))




to_groupby_SIMCOAL = ["popsize", 'starting_loci', 'chr_num', "chr_cM", "sample_size", 
    "genome_rep", "SIMCOAL_rep"]

grouped_SIMCOAL = summary_df.groupby(to_groupby_SIMCOAL, axis = 0)    
            
#determine best ylim
#global_ymin = 1
#global_ymax = 0
#for name, group in grouped:
#    local_ymin = np.min(group.mean_r2_unlinked)
#    local_ymax = np.max(group.mean_r2_unlinked)
#    if local_ymin < global_ymin: global_ymin = local_ymin
#    if local_ymax > global_ymax: global_ymax = local_ymax  
    
count= 0
mean_r2_values = list()
for name, group in grouped_SIMCOAL:
    #count+= 1
    #figure(count)
    #scatter(group.SIMCOAL_rep, group.mean_r2_unlinked, alpha = .5)
    #boxplot(group.mean_r2_unlinked)  
    #title(zip(to_groupby, name))
    #xlabel("SIMCOAL_rep")
    #ylabel("mean r^2 unlinked")
    #ylim(global_ymin * .98, global_ymax * 1.02)
    mean_r2_values.append(group.mean_r2_unlinked)
figure()
boxplot(x = mean_r2_values, notch = False, sym = "rD" )
xlabel("Replicate")
ylabel("mean r^2")




# One-way ANOVA test 
stats.f_oneway(*mean_r2_values)
stats.kruskal(*mean_r2_values)

# fitting a linear model 
from statsmodels.formula.api import ols, gls
#from statsmodels.graphics.api import interaction_plot, abline_plot
from statsmodels.stats.anova import anova_lm

# we can fit r2 from unlinked or all loci
# The effect of GENOME and SIMCOAL are larger with all loci (vs unlinked) as expected
# still need a way to properly nest SIMCOAL within GENOME, rather than onnly test one at a time.
# Even if effects are significant, they don't explain much of the variation.
# R2 is < .05, and the coefficients are on the order of 5e-05 


# first try to fit model with only the effect of GENOME
r2 = summary_df.mean_r2_all
genome = pd.Series([int(xx) for xx in summary_df.genome_rep], dtype = int)
formula = "r2 ~ C(genome)" # C() ensure they are treated categorically 
df_to_fit = pd.DataFrame({ 'r2': r2, 'genome' : genome})
lm = ols(formula, df_to_fit).fit()
print lm.summary()
anova_lm(lm, typ=2)

# Next try to fit a model to SIMCOAL runs within one of the GENOMES
# heacked to only consider the 1000 first 1000 values  (ie only GENOME rep 0)

r2 = summary_df.mean_r2_all[:1000]
coalescent = (["{}_{}".format(xx[1][0], xx[1][1]) for xx in summary_df[['genome_rep', 'SIMCOAL_rep']].iterrows()])[:1000] # effectively nesting SIMCOAL within genome rep
formula = "r2 ~ C(coalescent)" # C() ensure they are treated categorically 
df_to_fit = pd.DataFrame({ 'r2': r2, 'genome' : genome})[:1000]
lm = ols(formula, df_to_fit).fit()
print lm.summary()
anova_lm(lm, typ=2)

# OLD
r2 = summary_df.mean_r2_unlinked
#genome = pd.Series([int(xx) for xx in summary_df.genome_rep], dtype = object)d
genome = pd.Series([int(xx) for xx in summary_df.genome_rep], dtype = int)
#coalescent = pd.Series([(xx) for xx in summary_df[['genome_rep', 'SIMCOAL_rep']].iterrows()], dtype = object)
coalescent = (["{}_{}".format(xx[1][0], xx[1][1]) for xx in summary_df[['genome_rep', 'SIMCOAL_rep']].iterrows()]) # effectively nesting SIMCOAL within genome rep
# need one unique value for each unique [genome, simcoal] parameter combinationm
#coalescent = [count for count, val in enumerate(summary_df[['genome_rep', 'SIMCOAL_rep']].iterrows())] # effectively nesting SIMCOAL within genome rep

formula = "r2 ~ C(genome) + coalescent" # C() ensure they are treated categorically 

df_to_fit = pd.DataFrame({ 'r2': r2, 'genome' : genome, 'coalescent': coalescent})

lm = ols(formula, df_to_fit).fit()
lm_gls = gls(formula, df_to_fit).fit()

print lm.summary()


# Plotting residuals
resid = lm.resid
figure();
count = 0
for values, group in grouped_SIMCOAL:
    count+=1
    group_num = count
    x = [group_num] * len(group)
    plt.scatter(x, resid[group.index], alpha = .5)
plt.xlabel('Group')
plt.ylabel('Residuals')
xlim(0, count+1)





# with open(summary_file) as INFILE:
#pd.factorize([int(xx) for xx in summary_df.genome_rep])



# TESTING
# not sure if MultIndex is usful
summary_df_MULTINDEX = pd.DataFrame(summary_df)
to_groupby_MULTINDEX = ["popsize", 'starting_loci', 'chr_num', "chr_cM", "sample_size", 
    "genome_rep", "SIMCOAL_rep", "simuPOP_rep"]
grouped_MULTINDEX = summary_df_MULTINDEX.groupby(to_groupby_MULTINDEX, axis = 0)    
idx = pd.MultiIndex.from_tuples([name for name,group in grouped_MULTINDEX], names = to_groupby_MULTINDEX )
summary_df_MULTINDEX.index = idx
summary_df_MULTINDEX.index.names

# how to index: 
result_df = summary_df_MULTINDEX.loc[(summary_df_MULTINDEX.index.get_level_values('genome_rep') > 1) & (summary_df_MULTINDEX.index.get_level_values('popsize') == 400)]
