import collections
import numpy as np
from scipy import stats

def get_matings(matings_file):
    matings = []
    Mating = collections.namedtuple('Mating', ['gen', 'off', 'p1', 'p2'])
    with open(matings_file) as INFILE:
        for line in INFILE:
            gen, off, p1, p2 = [int(x) for x in line.strip().split()]
            one_mating = Mating(gen, off, p1, p2)
            matings.append(one_mating)
    return(matings)

def get_Ne_from_matings(matings):
    Ne_at_gen = dict()
    for generation in set([xx.gen for xx in matings]):
        parents = [xx.p1 for xx in matings if xx.gen == generation] + [xx.p2 for xx in matings if xx.gen == generation]
        N = len(parents)/2
        family_sizes = [collections.Counter(parents)[xx] for xx in range(N)] # vector of family sizes
        Ne_at_gen[generation] = (4. * N - 2) / (2+ np.var(family_sizes)) # crow 2010, equivalent to Waples 2011 eq 1
    return(Ne_at_gen)  
    
def Hmean_Ne_over_time(Ne_dict, generations):
    return(stats.hmean([Ne_dict[xx] for xx in generations]))

def variance_Ne_of_subsample(subsample, matings, generation = 9):
    parents = [xx.p1 for xx in matings if (xx.gen == generation and xx.off in subsample) ] + [xx.p2 for xx in matings if (xx.gen == generation and xx.off in subsample)]
    N = len(parents)/2
    family_sizes = [collections.Counter(parents)[xx] for xx in range(N)]
    Ne = (4. * N - 2) / (2+ np.var(family_sizes))
    return (Ne)
    
def sib_Ne_of_subsample(subsample, matings, generation = 9):
    #print(subsample)
    #print(matings)
    parents = [xx.p1 for xx in matings if (xx.gen == generation and xx.off in subsample) ] + [xx.p2 for xx in matings if (xx.gen == generation and xx.off in subsample)]
    N = len(parents)/2
    print(N)
    family_sizes = collections.Counter(parents).values()
    print(family_sizes)
    KsubIsq = np.sum([k**2 for k in family_sizes])
    print (KsubIsq)
    Ne = (2. * N - 1) / (KsubIsq/(2.* N) -1)
    return (Ne)
    

# How to use this
#my_matings_file = "C:/Data/sim_Ne/june-17/post_coalescent/matings/pop_50+loci_1024+chr_2+cM_100+cMunit_1000+ascern_01+genome_1-1+samp_NA+gen_NA+simupop_0+simcoal_0+samp_NA.matings"
#my_matings = get_matings(my_matings_file)
#my_Ne_dict = get_Ne_from_matings(my_matings)
#Hmean_Ne_over_time(my_Ne_dict, generations = [9]) # get Ne at generation 9
#Hmean_Ne_over_time(my_Ne_dict, generations = range(9)) # get Ne at generations 0-9





    
    
    

		