#!/usr/bin/env python
from __future__ import division

import os
import os.path
import sys

# PARAMETERS
from PARAMS import  BASE_DIR, REPORT_DIR, OMP_NUM_THREADS, PLINK_EXE, SIMCOAL_EXE, SIMCOAL_FORMAT, PDG_SPIDER_EXE, SPID, DEBUG, SCRATCH_DIR

# Simulation Settings
from simulation_settings import name, ploidy, map_function, mutation_rate, p_crit, fixed_families, popsizes, sampsizes, generations, SIMCOAL_min_allele_freqs, loci_num, chr_num, chr_cM, cM_unit, GENOME_replicates, SIMCOAL_replicates, simuPOP_replicates, sampling_replicates

# Logging
import logging
base_logger = logging.getLogger('')
base_logger.setLevel(logging.DEBUG)
formatter = logging.Formatter('%(asctime)s\t%(levelname)s\t%(name)s\t%(lineno)d\t%(message)s')

console_handler = logging.StreamHandler(sys.stderr)
console_handler.setFormatter(formatter)
console_handler.setLevel(logging.INFO)

logfile = os.path.join(REPORT_DIR, name, (name + '.log'))
file_handler = logging.FileHandler(logfile)
file_handler.setFormatter(formatter)
file_handler.setLevel(logging.DEBUG)

base_logger.addHandler(console_handler)
base_logger.addHandler(file_handler)

run_sim_log = logging.getLogger('run_sim')

print("Testing for report directory: {}".format(os.path.join(REPORT_DIR, name)))
if not os.path.exists(os.path.join(REPORT_DIR, name)):
    os.mkdir(os.path.join(REPORT_DIR, name))
    print("Created directory: {}".format(os.path.join(REPORT_DIR, name)))



# custom
from Sim_Batch_class import Sim_Batch
from plink_functions import batch_plink_r2
from results_functions import write_Sim_Report

run_sim_log.info("BASE_DIR = {}".format(BASE_DIR))
run_sim_log.info("REPORT_DIR = {}".format(REPORT_DIR))
run_sim_log.info("SCRATCH_DIR = {}".format(SCRATCH_DIR))
run_sim_log.info("DEBUG = {}".format(DEBUG))
run_sim_log.info("PLINK_EXE = {}".format(PLINK_EXE))
run_sim_log.info("SIMCOAL_EXE = {}".format(SIMCOAL_EXE))
run_sim_log.info("SIMCOAL_FORMAT = {}".format(SIMCOAL_FORMAT))
run_sim_log.info("PDG_SPIDER_EXE = {}".format(PDG_SPIDER_EXE))
run_sim_log.info("SPID = {}".format(SPID))
run_sim_log.info("OMP_NUM_THREADS = {}".format(OMP_NUM_THREADS))

#Check for and create necessary directories
run_sim_log.info("Testing for BASE_DIR: {}".format(BASE_DIR))
if not os.path.exists(BASE_DIR):
    run_sim_log.info("Created directory: {}".format(BASE_DIR))
    os.mkdir(BASE_DIR)
    
if not os.path.exists(SCRATCH_DIR):
    run_sim_log.info("Created directory: {}".format(SCRATCH_DIR))
    os.mkdir(SCRATCH_DIR)
    
if not os.path.exists(os.path.join(BASE_DIR, 'genomes')):
    run_sim_log.info("Created directory: {}".format(os.path.join(BASE_DIR, 'genomes')))
    os.mkdir(os.path.join(BASE_DIR, 'genomes'))
            
if not os.path.exists(os.path.join(BASE_DIR, 'coalescent')):
    run_sim_log.info("Created directory: {}".format(os.path.join(BASE_DIR, 'coalescent')))
    os.mkdir(os.path.join(BASE_DIR, 'coalescent'))
    
if not os.path.exists(os.path.join(BASE_DIR, 'post_coalescent')):
    run_sim_log.info("Created directory: {}".format(os.path.join(BASE_DIR, 'post_coalescent')))
    os.mkdir(os.path.join(BASE_DIR, 'post_coalescent'))
    
if not os.path.exists(os.path.join(BASE_DIR, 'post_coalescent', 'matings')):
    run_sim_log.info("Created directory: {}".format(os.path.join(BASE_DIR, 'post_coalescent', 'matings')))
    os.mkdir(os.path.join(BASE_DIR, 'post_coalescent', 'matings'))   
    
if not os.path.exists(os.path.join(BASE_DIR, 'samples')):
    run_sim_log.info("Created directory: {}".format(os.path.join(BASE_DIR, 'samples')))
    os.mkdir(os.path.join(BASE_DIR, 'samples'))
    
if not os.path.exists(os.path.join(BASE_DIR, 'samples', 'loci')):
    run_sim_log.info("Created directory: {}".format(os.path.join(BASE_DIR, 'samples', 'loci')))
    os.mkdir(os.path.join(BASE_DIR, 'samples', 'loci'))
    
if not os.path.exists(os.path.join(BASE_DIR, 'samples', 'individuals')):
    run_sim_log.info("Created directory: {}".format(os.path.join(BASE_DIR, 'samples', 'individuals')))
    os.mkdir(os.path.join(BASE_DIR, 'samples', 'individuals')) 
    
if not os.path.exists(os.path.join(BASE_DIR, 'samples', 'fold')):
    run_sim_log.info("Created directory: {}".format(os.path.join(BASE_DIR, 'samples', 'fold')))
    os.mkdir(os.path.join(BASE_DIR, 'samples', 'fold'))
    
if not os.path.exists(os.path.join(BASE_DIR, 'samples', 'genepop')):
    run_sim_log.info("Created directory: {}".format(os.path.join(BASE_DIR, 'samples', 'genepop')))
    os.mkdir(os.path.join(BASE_DIR, 'samples', 'genepop'))    
    
if not os.path.exists(os.path.join(BASE_DIR, 'samples', 'expected_r2')):
    run_sim_log.info("Created directory: {}".format(os.path.join(BASE_DIR, 'samples', 'expected_r2')))
    os.mkdir(os.path.join(BASE_DIR, 'samples', 'expected_r2'))
    
if not os.path.exists(os.path.join(BASE_DIR, 'samples', 'plink')):
    run_sim_log.info("Created directory: {}".format(os.path.join(BASE_DIR, 'samples', 'plink')))
    os.mkdir(os.path.join(BASE_DIR, 'samples', 'plink'))
    
if not os.path.exists(os.path.join(BASE_DIR, 'samples', 'plink', 'input')):
    run_sim_log.info("Created directory: {}".format(os.path.join(BASE_DIR, 'samples', 'plink', 'input')))
    os.mkdir(os.path.join(BASE_DIR, 'samples', 'plink', 'input'))
    
if not os.path.exists(os.path.join(BASE_DIR, 'samples', 'plink', 'output')):
    run_sim_log.info("Created directory: {}".format(os.path.join(BASE_DIR, 'samples', 'plink', 'output')))
    os.mkdir(os.path.join(BASE_DIR, 'samples', 'plink', 'output'))
    

my_sim_batch = Sim_Batch(     
        name = name,
       # Constants
        ploidy 			= ploidy,
        map_function 	        = map_function,
        mutation_rate 	        = mutation_rate,
        p_crit  		= p_crit,
        fixed_families         = fixed_families, 
        
        # Simulation Variables
        popsizes 		= popsizes,
        sampsizes 		= sampsizes,
        generations 	        = generations,
        SIMCOAL_min_allele_freqs = SIMCOAL_min_allele_freqs, 
        
        # Genome Variable
        loci_num		= loci_num,
        chr_num 		= chr_num,
        chr_cM 			= chr_cM,
        cM_unit 		= cM_unit,
        				
        #Replication
        GENOME_replicates 	= GENOME_replicates,
        SIMCOAL_replicates	= SIMCOAL_replicates,
        simuPOP_replicates 	= simuPOP_replicates,
        sampling_replicates     = sampling_replicates
)
    
run_sim_log.info('Simulation Batch Initialized')
run_sim_log.info("Simulation Parameter: name = {}".format(name)) 
run_sim_log.info("Simulation Parameter: ploidy = {}".format(ploidy)) 	
run_sim_log.info("Simulation Parameter: map_function = {}".format(map_function))  	
run_sim_log.info("Simulation Parameter: mutation_rate = {}".format(mutation_rate))  	
run_sim_log.info("Simulation Parameter: p_crit = {}".format(p_crit))   	
run_sim_log.info("Simulation Parameter: fixed_families = {}".format(fixed_families))   
run_sim_log.info("Simulation Parameter: popsizes = {}".format(popsizes))  	
run_sim_log.info("Simulation Parameter: sampsizes = {}".format(sampsizes))  	
run_sim_log.info("Simulation Parameter: generations = {}".format(generations))  	 
run_sim_log.info("Simulation Parameter: SIMCOAL_min_allele_freqs = {}".format(SIMCOAL_min_allele_freqs))
run_sim_log.info("Simulation Parameter: loci_num = {}".format(loci_num)) 	
run_sim_log.info("Simulation Parameter: chr_num = {}".format(chr_num))  	
run_sim_log.info("Simulation Parameter: chr_cM = {}".format(chr_cM)) 	
run_sim_log.info("Simulation Parameter: cM_unit = {}".format(cM_unit))  		
run_sim_log.info("Simulation Parameter: GENOME_replicates = {}".format(GENOME_replicates))  	
run_sim_log.info("Simulation Parameter: SIMCOAL_replicates = {}".format(SIMCOAL_replicates)) 
run_sim_log.info("Simulation Parameter: simuPOP_replicates = {}".format(simuPOP_replicates))
run_sim_log.info("Simulation Parameter: sampling_replicates = {}".format(sampling_replicates))

if __name__ == "__main__":
    run_sim_log.info('Simulate Data')
    my_sim_batch.run_sims(path = BASE_DIR)
    run_sim_log.info('Finished Running Simulations')
    
    # Run Plink to get R2 matrixes
    run_sim_log.info('Start Running PLINK')
    batch_plink_r2(os.path.join(BASE_DIR, 'samples', 'plink', 'input', "*.ped"), maf = .00, plink_exe = PLINK_EXE)
    run_sim_log.info('Finished Running PLINK')
    
    # Running Reports on simulated data
    run_sim_log.info('Start Running Report')
    write_Sim_Report(report_dir = os.path.join(REPORT_DIR, my_sim_batch.name), target_dir = os.path.join(BASE_DIR))
    run_sim_log.info('Finished Running Report')


# Can we leverage the temporal method in simulations to verify or backup our results
 

