#!/usr/bin/env python
from __future__ import division
import simuPOP 
import simuPOP.utils
import simuPOP.sampling

import os
import os.path
#import itertools
import numpy
#import subprocess
#import shlex

#import generate_genome  *
from PARAMS import DEBUG, BASE_DIR
#from generate_genome import expected_r2_SF, expected_r2_WH

import logging
simuPOP_functions_log = logging.getLogger('simuPOP_functions')


def write_parents(pop, param):
    generation = pop.vars()['gen']
    simuPOP_functions_log.debug(("Exporting parents, generation: {}".format(generation)))

    try:
	popsize = param['popsize']
	ascertainment_AF = param['ascertainment_AF']
	loci_num = param['loci_num']
	chr_num = param['chr_num']
	chr_cM = param['chr_cM']
	rep = param['rep']
	SIMCOAL_rep = param['SIMCOAL_rep']
	cM_unit = param['cM_unit']
	current_genome = param['current_genome']
	sampling_rep = "NA"
	sample_size = "NA" # no samples taken yet
    except KeyError:
	print("Can't Unpack Parameters")
	raise Exception("Can't unpack params")    
    name_tuple = (popsize,
			loci_num,
			chr_num,
			chr_cM,
			cM_unit,
			str(ascertainment_AF).replace('.', ''),
			"-".join([str(jj) for jj in current_genome]), # current_genome
			sample_size, 
			"NA", # gen
			rep, # simupop_rep
			SIMCOAL_rep, 
			sampling_rep
    ) 
    simuPOP_name = "pop_{}+loci_{}+chr_{}+cM_{}+cMunit_{}+ascern_{}+genome_{}+samp_{}+gen_{}+simupop_{}+simcoal_{}+samp_{}".format(*name_tuple)
    with open(os.path.join(BASE_DIR, "post_coalescent", "matings", "{}.matings".format(simuPOP_name)), 'a') as OUTFILE:
        for ind_id in range(popsize):
            OUTFILE.write("{}\t{}\t{}\t{}\n".format(generation, ind_id, [int(x) for x in pop.indInfo('mother_idx')][ind_id], [int(x) for x in pop.indInfo('father_idx')][ind_id]))
    return(True)            

def process_subsample(pop, param):
	generation = pop.vars()['gen']
	#if DEBUG: print("Processing subsample {}".format(param))
	simuPOP_functions_log.debug(("Processing subsample {}".format(param)))
	pop.addInfoFields('ind_id')
	try:
		samp_sizes = param['samp_sizes'] 
		Genome = param['Genome']
		write_WH_SF_exp_r2 = param['write_WH_SF_exp_r2']
		write_genepop = param['write_genepop']
		write_map_ped = param['write_map_ped']
		path = param['path']		
		popsize = param['popsize']
		ascertainment_AF = param['ascertainment_AF']
		loci_num = param['loci_num']
		chr_num = param['chr_num']
		chr_cM = param['chr_cM']
		rep = param['rep']
		SIMCOAL_rep = param['SIMCOAL_rep']
		cM_unit = param['cM_unit']
		current_genome = param['current_genome']
		sampling_replicates = param['sampling_replicates']
	except KeyError:
		print("Can't Unpack Parameters")
		raise Exception("Can't unpack params")
	#if DEBUG: print("Finished unpacking params")
	simuPOP_functions_log.debug("Finished unpacking params")
	for sample_size in samp_sizes:
		if sample_size <= pop.popSize():
		    for sampling_rep in range(sampling_replicates):
		        sampling_rep += 1 # match 1-based indexing
		        
		        #simuPOP.IdTagger.reset(startID= 0)
		        simuPOP.tagID(pop, reset = 1)
			
			subpop = simuPOP.sampling.drawRandomSample(pop, sample_size)
			simuPOP_functions_log.debug("Constructing name")
			name_tuple = (popsize,
				loci_num,
				chr_num,
				chr_cM,
				cM_unit,
				str(ascertainment_AF).replace('.', ''),
				"-".join([str(jj) for jj in current_genome]), # current_genome
				sample_size, 
				generation, 
				rep, # simupop_rep
				SIMCOAL_rep, 
				sampling_rep
				) 
			simuPOP_name = "pop_{}+loci_{}+chr_{}+cM_{}+cMunit_{}+ascern_{}+genome_{}+samp_{}+gen_{}+simupop_{}+simcoal_{}+samp_{}".format(*name_tuple)
			full_path = os.path.join(path, 'samples', simuPOP_name)
			simuPOP_functions_log.debug("Name: {}".format(simuPOP_name))
			simuPOP_functions_log.debug("Filter_MAF()")
			#loci_to_keep, loci_to_remove = filter_MAF(subpop, maf = 0.05)
			loci_to_keep, loci_to_remove = filter_MAF(subpop, maf = 0.00)
			simuPOP_functions_log.debug("keeping {} loci".format(len(loci_to_keep)))
			#simuPOP_functions_log.debug("excluding {} loci".format(len(loci_to_remove)))
			if (len(loci_to_keep) + len(loci_to_remove)) != loci_num:
				raise ValueError("Unexpected loci_num: {}".format(loci_num))
			
			# write included and excluded loci to files		
			with open(os.path.join(BASE_DIR, 'samples', "loci", "{}.include".format(simuPOP_name)), 'w') as OUTFILE:
			    OUTFILE.write("\n".join([str(xx) for xx in loci_to_keep]))
			with open(os.path.join(BASE_DIR, 'samples', "loci", "{}.exclude".format(simuPOP_name)), 'w') as OUTFILE:
			    OUTFILE.write("\n".join([str(xx) for xx in loci_to_remove]))
			
			## RW 6-17
			with open(os.path.join(BASE_DIR, 'samples', "individuals", "{}.subsample".format(simuPOP_name)), 'w') as OUTFILE:
			    OUTFILE.write("\n".join([str(int(ind.ind_id)-1) for ind in subpop.individuals()]))
						

			simuPOP_functions_log.debug("Start export genepop, file = {}".format(full_path + ".gen"))
			if write_genepop is True:
				simuPOP.utils.export(subpop, format = 'genepop', output = os.path.join(path, 'samples', 'genepop', "{}.gen".format(simuPOP_name)), adjust = 1)

			simuPOP_functions_log.debug("Created genepop file: {}".format(os.path.join(path, 'samples', 'genepop', "{}.gen".format(simuPOP_name))))
			simuPOP_functions_log.debug("write_map_ped: {}".format(write_map_ped))
			if write_map_ped is True:
				simuPOP.utils.export(subpop, format = 'MAP', output = os.path.join(path, 'samples', 'plink', 'input', "{}.map".format(simuPOP_name)))
				simuPOP.utils.export(subpop, format = 'PED', output = os.path.join(path, 'samples', 'plink', 'input', "{}.ped".format(simuPOP_name)),  adjust = 1)
		
			simuPOP_functions_log.debug("Created MAP PED files: {}".format(os.path.join(path, 'samples', 'plink', 'input', "{}.*".format(simuPOP_name))))
			
			# 1/8/14
			# Re-evaluate this below here:
			# The WH & SF & lower r2 calculations moved to analysis.
			# Stopped printing the c matrix for each rep.
								
			# generate folded allele freq spectrum
			freqs = get_folded_freqs(subpop)
			with open(os.path.join(BASE_DIR, 'samples', "fold", "{}.fold".format(simuPOP_name)), 'w') as OUTFILE:
				OUTFILE.write("\n".join(str(xx) for xx in freqs))
				simuPOP_functions_log.debug("Folded Allele Frequencies written to file: {}".format(os.path.join(BASE_DIR, 'samples', "fold", "{}.fold".format(simuPOP_name))))
	return (True)
	
def filter_MAF(pop, missing_allele_code = None, maf = 0.05, DEBUG = DEBUG):
	simuPOP.stat(pop, alleleFreq = simuPOP.ALL_AVAIL)
	loci_to_keep = []
	loci_to_remove = []
	for locus in pop.vars()['alleleNum'].keys():
		allele_counts = []
		for allele in pop.vars()['alleleNum'][locus]:
			if allele != missing_allele_code:
				allele_counts.append(pop.vars()['alleleNum'][locus][allele])
		if len(allele_counts) > 2:
			raise StandardError("Should not be more than two alleles at locus {}".format(locus))
		allele_ratio = allele_counts[0] / float(sum(allele_counts) )
		if (allele_ratio > 1 - maf) or (allele_ratio < maf):
			#simuPOP_functions_log.debug("{}, {}, {}".format(locus, allele_counts, allele_ratio))
			loci_to_remove.append(locus)
		else:
			loci_to_keep.append(locus)

	simuPOP_functions_log.debug("Removing {} loci for MAF < {}:".format(len(loci_to_remove), maf))
	simuPOP_functions_log.debug(str(loci_to_remove))
		
	pop.removeLoci(loci = loci_to_remove)
	return(loci_to_keep, loci_to_remove)		
								
def read_R2_matrix(matrix_file, bound_at_one = True):
	"""Can natively read *.gz files"""
	try:
	    R2 = numpy.load(matrix_file)
	except IOError as e:
	   simuPOP_functions_log.debug("I/O error({0}): {1}".format(e.errno, e.strerror))
	   simuPOP_functions_log.debug("trying numpy.loadtxt()")
	   R2 = numpy.loadtxt(matrix_file)
	# limit values to 1
	if bound_at_one:
	    R2[R2 > 1] = 1
	# collapse to 1D vector 
	R2 = R2[numpy.triu_indices_from(R2, k = 1)]
	return(R2)

	
def get_folded_freqs(pop):
	simuPOP.stat(pop, alleleFreq = simuPOP.ALL_AVAIL)
	freq_spectrum = []
	for loc in pop.vars()['alleleFreq'].keys():
		max_freq = max(list(pop.vars()['alleleFreq'][loc].values()))
		freq_spectrum.append(max_freq)
	folded_freq_spectrum = [ii if ii<=.5 else 1-ii for ii in freq_spectrum]
	return(folded_freq_spectrum)
