#!/usr/bin/env python


# def parse_filename(filename):
	# """Parses a sfs_code filename base into a dictionary of the parameters specified by the name."""
	# root = os.path.splitext(os.path.basename(filename))[0]
	# print(root)
	# replicate, params = root.split('.sim_')
	# parameters = dict(zip(('rep', 'popsize', 'sampSize', 'BURN', 'afterBURN', 'rho', 'BURN_theta', 'afterBURN_theta', 'loci', 'num_chrs', 'len_chrs'), [replicate] + (params.split('_'))))
	# return(parameters)


	
# taken from:
# http://stackoverflow.com/questions/136168/get-last-n-lines-of-a-file-with-python-similar-to-tail 
def tail( f, window):
    BUFSIZ = 1024
    f.seek(0, 2)
    bytes = f.tell()
    size = window
    block = -1
    data = []
    while size > 0 and bytes > 0:
        if (bytes - BUFSIZ > 0):
            # Seek back one whole BUFSIZ
            f.seek(block*BUFSIZ, 2)
            # read BUFFER
            data.append(f.read(BUFSIZ))
        else:
            # file too small, start from begining
            f.seek(0,0)
            # only read what was not read
            data.append(f.read(bytes))
        linesFound = data[-1].count('\n')
        size -= linesFound
        bytes -= BUFSIZ
        block -= 1
    return '\n'.join(''.join(data).splitlines()[-window:])	
	
	



def time_to_fixation(Ne, initial_freq = .5):
	p = initial_freq
	return(-4*Ne*(1-p)*numpy.log(1-p)/p)
	
def var_all_freq(p, Ne, t):
	q = 1.-p
	return(p*q*(1- numpy.exp(-(t/(2*Ne)))))
	


def advance_n_gen_Park(Ne, c, initial_r2, n):
	"""
	Park
	Effective population size of current human population
	"""
	for nn in range(n):
		final_r2 = (initial_r2 + 1./(2*Ne)) * (1-c)**2 + 1./(2*Ne)
		initial_r2 = final_r2
	return(final_r2)
	
	
	return(final_r2)

def Q(Ne, c, initial_r2):
	"""
	Park L (2012) Linkage Disequilibrium Decay and Past Population History in the Human Genome. 
	PLoS ONE 7(10): e46603. doi:10.1371/journal.pone.0046603
	"""
	a = 1./(2*Ne)
	b1 = 1-(1./(2*Ne))
	b2 = (1-c)**2
	b3 = initial_r2
	return(a + b1*b2*b3)
	
	
def advance_r2_n_generations(Ne, c, n, ini_r2 = 1, selfing = False):
	Ne = numpy.int(Ne)
	c = numpy.float(c) 
	ini_r2 = numpy.float(ini_r2)
	n = int(n)
	final_r2 = ini_r2
	for xx in range(n):
		final_r2 = advance_r2_one_generation(Ne = Ne, c = c, ini_r2 = ini_r2, selfing = selfing)
		ini_r2 = final_r2
	return(final_r2)

def generations_to_equilibrium_composite(Ne, min_c, max_gen = 1000, threshold= .0001, ini_r2 = 1., selfing = True):
	c = float(min_c)
	ini_r2 = float(ini_r2)
	max_gen = int(max_gen)

	expected_Park = expected_r2_Park(Ne = Ne, c = c)
	expected_WH = expected_r2_WH(Ne = Ne, c = c, reshape = False)
	expected_SF = expected_r2_SF(Ne = Ne, c = c, reshape = False)
	eq_r2 = min(expected_Park, expected_WH, expected_SF)
	index = [expected_Park, expected_WH, expected_SF].index(eq_r2)
	method = ["Park", "WH", "SF"][index]
	n = range(max_gen)
	r2_at_gen = list()
	for xx in n:
		r2_at_gen.append(advance_r2_n_generations(Ne, c = min_c, ini_r2 = ini_r2, n = xx, selfing = True))
	r2_at_gen = numpy.array(r2_at_gen)
	if ini_r2 > eq_r2:
		gen_within_threshold = r2_at_gen <= eq_r2 + threshold
	else:
		gen_within_threshold = r2_at_gen >= eq_r2 - threshold
	if sum(gen_within_threshold) == 0: # what if we predict we never meet it
		# find generation where it is changing by less than threshold per gen
		diffs = numpy.diff(r2_at_gen)
		#print(diffs)
		first_gen_within_threshold = numpy.where(-diffs < (threshold))[0][0]
		
	# get first generation where this is true:	
	else:
		first_gen_within_threshold = numpy.where(gen_within_threshold == True)[0][0]
	return(first_gen_within_threshold, eq_r2, method)	
	
def advance_r2_one_generation(Ne, c, ini_r2 = 1, selfing = False):
	assert(Ne == int(Ne))
	Ne = numpy.float(Ne)
	c = numpy.float(c) 
	ini_r2 = numpy.float(ini_r2)
	_1a = (1-(c/Ne))
	#print( _1a)
	_1b = (1-(1./(2*Ne)))
	#if selfing == False:
	#	_1b = 1
	#print( _1b)
	_1c = (1-c)**2
	#print( _1c)
	_2 = (c**2)/ (2*Ne)
	if selfing == False:
		_2 = 0
	#print( _2)
	_3 = 1./(2*Ne)
	#print( _3)
	final_r2 = ini_r2 * _1a * _1b * _1c + _2 + _3
	return (final_r2)

def generations_to_equilibrium_Park( Ne, c, initial_r2 =1):
	"""
	Taken from equation 3 in:
	Park L (2012) Linkage Disequilibrium Decay and Past Population History in the Human Genome. 
	PLoS ONE 7(10): e46603. doi:10.1371/journal.pone.0046603
	"""
	Ne = float(Ne)
	initial_r2 = float(initial_r2)
	c = float(c)
	top_1 = -numpy.log(2 * Ne) 
	top_2a = initial_r2
	top_2b = 1/(2*Ne)
	top_2c_top = 1 + c**2
	top_2c_bot = 1 - (1-(c/Ne)) * (1 - (1/(2*Ne))) * (1 - c)**2
	top_2c = top_2c_top / top_2c_bot
	top2 = top_2a - (top_2b * top_2c)
	top = top_1 - numpy.log(top2)
	bot_1 = numpy.log(1 - (c/Ne))
	bot_2 = numpy.log(1 - (1/(2*Ne)))
	bot_3 = 2 * numpy.log(1-c)
	bot = bot_1 + bot_2 + bot_3
	#print(top, bot)
	n = top / bot
	return(n)
	
def generations_to_equilibrium_composite(Ne, min_c, max_gen = 1000, threshold= .0001, ini_r2 = 1.):
	c = float(min_c)
	ini_r2 = float(ini_r2)
	max_gen = int(max_gen)

	expected_Park = expected_r2_Park(Ne = Ne, c = c)
	expected_WH = expected_r2_WH(Ne = Ne, c = c, reshape = False)
	expected_SF = expected_r2_SF(Ne = Ne, c = c, reshape = False)
	eq_r2 = min(expected_Park, expected_WH, expected_SF)
	index = [expected_Park, expected_WH, expected_SF].index(eq_r2)
	method = ["Park", "WH", "SF"][index]
	
	n = range(max_gen)
	r2_at_gen = list()
	for xx in n:
		r2_at_gen.append(advance_r2_n_generations(500, c =.01, ini_r2 = ini_r2, n = xx))
	r2_at_gen = numpy.array(r2_at_gen)
	if ini_r2 > eq_r2:
		gen_within_threshold = r2_at_gen <= eq_r2 + threshold
	else:
		gen_within_threshold = r2_at_gen >= eq_r2 - threshold
	if sum(gen_within_threshold) == 0: # what if we predict we never meet it
		# find generation where it is changing by less than threshold/5 per gen
		diffs = numpy.diff(r2_at_gen)
		#print(diffs)
		first_gen_within_threshold = numpy.where(-diffs < (threshold / 5.))[0][0]
		
	# get first generation where this is true:	
	else:
		first_gen_within_threshold = numpy.where(gen_within_threshold == True)[0]
	return(first_gen_within_threshold, eq_r2, method)	
	
		
	
	
	
	n = generations_to_equilibrium_Park(Ne = Ne, c = c, initial_r2 = ini_r2)
	return(n, eq_r2, method)

def r2_after_n(Ne, c, n, ini_r2 = 1.):
	"""	
	n = generations
	Taken from equation 2 in:
	Park L (2012) Linkage Disequilibrium Decay and Past Population History in the Human Genome. 
	PLoS ONE 7(10): e46603. doi:10.1371/journal.pone.0046603
	"""
	Ne = float(Ne)
	c = float(c)
	n = int(n)
	ini_r2 = float(ini_r2)
	
	first_term = (1-(c/Ne))**n * (1-(1/(2*Ne)))**n * (1-c)**(2*n) * ini_r2 
	#top
	second_term_top = 1 - (1-(c/Ne))**n * (1-(1/(2*Ne)))**n * (1-c)**(2*n)
	#bot
	second_term_bot_a = 1-(c/Ne)
	second_term_bot_b = 1-(1/(2*Ne))
	second_term_bot_c = (1-c)**2
	second_term_bot = 1 - second_term_bot_a * second_term_bot_b * second_term_bot_c
	last = (1 + c**2)/(2*Ne)
	second_term = (second_term_top/second_term_bot) * last
	#print("first_term: {}\nsecond_term: {}\n\tsecond_term_top {}\t second_term_bot {}\t last {}".format(first_term, second_term, second_term_top, second_term_bot, last))
	final = first_term + second_term
	return(final)
	

def advance_r2_one_generation(Ne, c, ini_r2 = 1, selfing = False):
	Ne = numpy.int(Ne)
	c = numpy.float(c) 
	ini_r2 = numpy.float(ini_r2)
	
	_1a = (1-c/Ne)
	#print( _1a)
	_1b = (1-1./(2*Ne))
	if selfing == False:
		_1b = 1
	#print( _1b)
	_1c = (1-c)**2
	#print( _1c)
	_2 = (c**2)/ (2*Ne)
	if selfing == False:
		_2 = 0
	#print( _2)
	_3 = 1./(2*Ne)
	#print( _3)
	final_r2 = ini_r2 * _1a * _1b * _1c + _2 + _3
	return (final_r2)
	
def advance_r2_n_generations(Ne, c, n, ini_r2 = 1, selfing = False):
	Ne = numpy.int(Ne)
	c = numpy.float(c) 
	ini_r2 = numpy.float(ini_r2)
	n = int(n)
	final_r2 = ini_r2
	for xx in range(n):
		final_r2 = advance_r2_one_generation(Ne = Ne, c = c, ini_r2 = ini_r2, selfing = selfing)
		ini_r2 = final_r2
	return(final_r2)
	
	
def eval_index_matrix(SR, index_mat):
	obs_r2 = SR.r2_plink[index_mat]
	exp_r2 = SR.lower_r2_estimate[index_mat]
	c = SR.c_mat[index_mat]
	return(obs_r2, exp_r2, c)

	with open('C:/Users/waplesry/Dropbox/Projects/simulate_Ne/sim_analysis/batch_10/results.tsv', 'a') as RESULTSFILE:
		RESULTSFILE.write("{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t".format('name', 'popsize', 'sample_size', 'generation', 'loci_num', 'chr_num', 'chr_cM', 'scale_recombination', 'scale_mutation', 'rep'))
		RESULTSFILE.write("{}\t{}\t{}\t{}\t".format('mean_obs_r2', 'mean_exp_r2', 'final_loci', 'meanAF'))
		RESULTSFILE.write("{}\t{}\t{}\t{}\t".format(str(.5) + "_obs_r2", str(.5) + "_exp_r2", str(.5) + "_c" , str(.5) + "_len" ))
		RESULTSFILE.write("{}\t{}\t{}\t{}\t".format(str(.1) + "_obs_r2", str(.1) + "_exp_r2", str(.1) + "_c" , str(.1) + "_len" ))
		RESULTSFILE.write("{}\t{}\t{}\t{}\n".format(str(.01) + "_obs_r2", str(.01) + "_exp_r2", str(.01) + "_c" , str(.01) + "_len" ))
		
	for gen_file in glob.glob("C:/DATA/simuPOP/batch_10/*.gen"):
		dir = os.path.dirname(gen_file)
		filename = os.path.basename(gen_file)
		base_name = os.path.splitext(filename)[0]
		print(dir, base_name)
		SR = Sim_Results(dir, base_name)
		SR.set_index_array_c(.5, 0.49999, 0.50001)
		SR.set_index_array_c(.1, 0.09, 0.11)
		SR.set_index_array_c(.01, 0, 0.01)
		subset_dict = dict()
		for k, v in SR.subset_loci_by_c.items():
			obs_r2, exp_r2, c = eval_index_matrix(SR = SR, index_mat = v)
			
			subset_dict[k] = mean(obs_r2), mean(exp_r2), mean(c), len(c)
		
		with open('C:/Users/waplesry/Dropbox/Projects/simulate_Ne/sim_analysis/batch_10/results.tsv', 'a') as RESULTSFILE:
				RESULTSFILE.write("{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t".format(SR.name, SR.popsize, SR.sample_size, SR.generation, SR.loci_num, SR.chr_num, SR.chr_cM, SR.scale_recombination, SR.scale_mutation, SR.rep))
				RESULTSFILE.write("{}\t{}\t{}\t{}\t".format(SR.mean_obs_r2, SR.mean_exp_r2, SR.final_loci, SR.meanAF))
				RESULTSFILE.write("{}\t{}\t{}\t{}\t".format(subset_dict[.5][0], subset_dict[.5][1], subset_dict[.5][2], subset_dict[.5][3]))
				RESULTSFILE.write("{}\t{}\t{}\t{}\t".format(subset_dict[.1][0], subset_dict[.1][1], subset_dict[.1][2], subset_dict[.1][3]))
				RESULTSFILE.write("{}\t{}\t{}\t{}\n".format(subset_dict[.01][0], subset_dict[.01][1], subset_dict[.01][2], subset_dict[.01][3]))

				
# def generations_to_equilibrium(initial_r2, Ne, c):
	# top_1 = -numpy.log(2 * Ne) 
	# top_2a = initial_r2
	# top_2b = 1/(2*Ne)
	# top_2c_top = 1 + c**2
	# top_2c_bot = 1 - (1-(c/Ne)) * (1 - (1/(2*Ne))) * (1 - c)**2
	# top_2c = top_2c_top / top_2c_bot
	# top2 = top_2a - (top_2b * top_2c)
	# top = top_1 - numpy.log(top2)
	
	# bot_1 = numpy.log(1 - (c/Ne))
	# bot_2 = numpy.log(1 - (1/2*Ne))
	# bot_3 = 2 * numpy.log(1-c)
	# bot = bot_1 + bot_2 + bot_3
	
	# n = top / bot
	# return(n)				
		
def plot_chi2pdf(df, scale):

	x = numpy.linspace(0.00001, 1, 500)
	#chi = scipy.stats.chi2(df=1)
	z = scipy.stats.chi2(x, df = 1)
	y = scipy.stats.chi2.pdf(x, df = 1), loc=0, scale = 1)
	#y = chi.pdf(x)

	pyplot.plot(x, y)
	pylab.xlim([-0.04, 1.04])
	#, bins = 500, log = True, range = (0, 1))
	plt.xlabel("X")
	plt.ylabel("Probability")
	plt.draw()