#!/usr/bin/env python
from __future__ import division
import random
import math
import itertools
import collections
import scipy.spatial.distance
import numpy

# # Placing loci onto a chromosome
# def generate_cM_chromosome(cM, num_loci, unit):
	# """
	# Returns a list of *num_loci* loci onto a chromosome *cM* units long.
	# Positions are given as cM postions multiplied by unit.
	# """
	# if cM <= 0:
		# raise Exception("Please select a postive cM length chromosome")
	# else:
		# positions = sorted(random.sample(range(int(cM * unit)), num_loci))
		# cM_chromosome = [xx for xx in positions]
		# return (cM_chromosome)
		
# def c_adjacent(cM_chromosome, unit, map_function = "kosambi"):
	# """
	# Returns a list recombination fractions between adjacent loci on the supplied chromosome.
	# 'kosambi' is the only currently supported map function.
	# """
	# locus_spacings = [(jj-ii) / unit for ii, jj in zip(cM_chromosome[:-1], cM_chromosome[1:])]
	# r_fractions = [cM_to_c(xx, map_function) for xx in locus_spacings]
	# return (r_fractions)


# def generate_genome_simuPOP(chrom_num, chrom_cM, chrom_loci, unit, map_function = 'kosambi'):
	# """
	# Returns a tuple of nested lists.
		# [0] = to be passed to simuPOP as marker positions.  
			# These are cumulative, pariwise recombination fractions.  
			# To be used as locus positions by simuPOP.  The recombination frequency between adjacent loci is scaled by this distance.
		# [1]	= These are int marker positions in unit * cM.  Loci cannot be at the same position.  
			# Max number of unique positions  = cM of chromosome * unit
	# Initial SNP on each chromosome is assigned a postion of 0.
	# """
	# if chrom_cM == 0:
		# c_genome = []
		# for xx in range(chrom_num):
			# c_chromosome = [xx * 0.5 for xx in range(chrom_loci)]
			# c_genome += [c_chromosome]
		# cM_genome = None
	# else:
		# c_genome = []
		# cM_genome = []
		# for ii in range(chrom_num):
			# cM_chromosome = generate_cM_chromosome(cM = chrom_cM, num_loci = chrom_loci, unit = unit)
			# cM_genome += [cM_chromosome]
			# c_chromosome = numpy.cumsum([0.0] + c_adjacent(cM_chromosome, unit, map_function))
			# c_genome += [c_chromosome.tolist()] #converted back to list here
	# return(c_genome, cM_genome)
	
# ## generate recombination fraction (c) between all pairs of loci on the same chromosome.
# def chromosome_c_matrix(cM_chromosome, unit, map_function = "kosambi"):
	# """
	# Pairwise recombination fraction (c) calculated for a single linear chromosome.  
	# Returns a square distance matrix
	# """
	# point_matrix = numpy.matrix([[xx/unit] for xx in cM_chromosome]) #convert back to cM from cM*unit
	# condensed_dist_matrix = scipy.spatial.distance.pdist(point_matrix)
	# condensed_rfrac_matrix = .5 * numpy.tanh(2 * condensed_dist_matrix/100.0)
	# sqaure_matrix = scipy.spatial.distance.squareform(condensed_rfrac_matrix)
	# return(sqaure_matrix)
	
# def genome_c_matrix(cM_genome, num_loci, unit):
	# if cM_genome is None:
		# total_loci = num_loci
		# initial_matrix = numpy.array([0.5 for xx in range(total_loci**2)])
		# initial_matrix = numpy.reshape(a = initial_matrix, newshape = (total_loci, total_loci))
		# return(initial_matrix)
	# else:
		# cM_positions = numpy.array(cM_genome)/float(unit)
		# total_loci = len(cM_positions.flatten())
		# initial_matrix = numpy.array([0.5 for xx in range(total_loci**2)])
		# initial_matrix = numpy.reshape(a = initial_matrix, newshape = (total_loci, total_loci))
		# start_pos = 0 
		# last_pos = 0
		# for each_chr in range(len(cM_genome)):
			# partial_c_matrix = chromosome_c_matrix(cM_genome[each_chr], unit = unit)
			# last_pos = start_pos + len(cM_genome[each_chr])
			# initial_matrix[start_pos:last_pos,start_pos:last_pos] = partial_c_matrix
			# start_pos = last_pos
		# return(initial_matrix)
	
	
def cM_to_c(cM, map_function):
	"""Takes as input a cM distance and a map function ["kosambi", "haldane", "etc.."] 
	and returns the recombination fraction (c) as calculated by the map_function.
	Currently only map_function = "kosambi" is supported.
	Kosambi Citation:
		Genetics. 1996 Apr;142(4):1369-77.
		On genetic map functions.
		Zhao H, Speed TP."""
	if map_function == "kosambi":
		r_frac = (.5) * numpy.tanh(2*cM/100.0)
		return (r_frac)
	else:
		raise Exception("Only kosambi is supported")	

def est_c_numerator(c):
	"""
	(Matrix-aware)  
	Solves the expression:
	((1-c)**2 + c**2)/(4*c - 2*c**2)
	for the supplied recombination fraction (c).
	numerator from  Weir & Hill 1981
	"""
	# c = recombination fraction
	# return numerator from  Weir & Hill 1981
	# (r2 - (1/S)) * Ne = ((1-c)**2 + c**2)/(4*c - 2*c**2)
	#if c is 0:
	#	return(inf)
	#else:
	return (((1-c)**2 + c**2)/(4*c - 2*c**2))
	
def expected_r2_WH(c, Ne, bound = True, reshape = True):
	"""
	Weir and Hill 1981?
	c - recombination fraction
	Ne - Effective Size
	S - Sample size
	And returns the projected r^2 value.
	"""
	numerator = est_c_numerator(c)
	est_r2 = (numerator / Ne)
	if reshape == True:
		if c.__class__ == numpy.ndarray:
			numpy.fill_diagonal(est_r2, 1)
			if bound:
				est_r2[est_r2 > 1] = 1
	return est_r2
	
def expected_r2_SF(c, Ne, bound = True, reshape = True):
	"""
	Sved and Feldman (1973)
	c - recombination fraction
	Ne - Effective Size
	S - Sample size
	And returns the projected r^2 value.
	"""
	est_r2 = 1./(1 + 2 * Ne * c *(2 - c))
	if reshape == True:
		if c.__class__ == numpy.ndarray:
			numpy.fill_diagonal(est_r2,1)
			if bound:
				est_r2[est_r2 > 1] = 1
	return est_r2
	
def expected_r2_Park(c, Ne, bound = True, reshape = True):
	"""
		Citation:
		In text on page 2 of:
		Park L (2012) 
		Linkage Disequilibrium Decay and Past Population History in the Human Genome. 
		PLoS ONE 7(10): e46603. 
		doi:10.1371/journal.pone.0046603
	"""
	if c.__class__ == int:
		c = float(c)
	Ne = float(Ne)
	top = 1 + c**2
	bot = (1 - (1 - 1/(2*Ne)) * (1 -(c/Ne)) * (1-c)**2) * 2*Ne
	est_r2 = top/bot
	if reshape == True:
		if c.__class__ == numpy.ndarray:
			numpy.fill_diagonal(est_r2,1)
			if bound:
				est_r2[est_r2 > 1] = 1
	return(est_r2)	
